@@ -7,10 +7,7 @@ BEGIN { eval { require warnings } ? 'warnings'->import : ( $^W = 1 ) }
 use Carp;
 use Symbol;
 
-$VERSION = '1.01';
-
-binmode STDERR;
-select( ( select(STDERR), $| = 1 )[0] );
+$VERSION = '1.0';
 
 sub new {
     my ($class) = @_;
@@ -31,7 +28,7 @@ sub new {
 sub DESTROY {
     my ($duplicate) = @_;
 
-    open STDERR, '>&=' . fileno $duplicate
+    open STDERR, '<&=' . fileno $duplicate
         or croak 'Failed to return STDERR';
 
     unlink 't/STDERR.test';
@@ -44,7 +41,6 @@ sub get {
     my $stderr = gensym;
     open $stderr, '<t/STDERR.test'
         or croak 'Failed to open file piped from STDERR';
-    binmode $stderr;
 
     return do { local $/; <$stderr> };
 }
